#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

import numpy
from bob.pad.base.algorithm import Algorithm

import bob.learn.em
import bob.io.base

class OCGMM(Algorithm):
  """
  This class interfaces a GMM-based "classifier" to perform PAD experiments

  A GMM is used to model the bonafide features

  """

  def __init__(self, n_gaussians=2, max_iter=1000, conv_threshold=1e-5, **kwargs):

    Algorithm.__init__(self,  
                       performs_projection=True,
                       requires_projector_training=True, 
                       **kwargs)

    self.n_gaussians = n_gaussians
    self.max_iter = max_iter
    self.conv_threshold = conv_threshold

    self.machine = None
    self.trainer = bob.learn.em.ML_GMMTrainer(update_means=True, update_variances=True, update_weights=True)


  def train_projector(self, training_features, projector_file):
    """
    Trains the GMM using Expectation-Maximimazation with Maximum Likelihood criterion 

    **Parameters**

    training_features:
    """
    # training_features[0] - training features for the REAL class.
    # training_features[1] - training features for the ATTACK class.
    
    # The data - "positive class only"
    pos = numpy.array(training_features[0])
    features_dim = pos.shape[1]
   
    # The machine
    self.machine = bob.learn.em.GMMMachine(self.n_gaussians, features_dim)

    # train
    bob.learn.em.train(self.trainer, self.machine, pos, max_iterations=self.max_iter, convergence_threshold=self.conv_threshold)
    f = bob.io.base.HDF5File(projector_file, 'w')
    self.machine.save(f)
    

  def project(self, feature):
    """
    Compute the log-likelihood of the feature 
    """
    return self.machine(feature)

  def score(self, toscore):
    return [toscore[0]]
