#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

import numpy
from bob.pad.base.algorithm import Algorithm

import bob.learn.libsvm
import bob.io.base

class OCSVM(Algorithm):
  """
  This class interfaces a One Class SVM classifier used for PAD

  """

  def __init__(self, rescale=True, nu=0.01, gamma=0.1, **kwargs):

    Algorithm.__init__(self,  
                       performs_projection=True,
                       requires_projector_training=True, 
                       **kwargs)

    self.rescale = rescale 
    self.nu = nu
    self.gamma = gamma
    self.machine = None
    self.trainer = bob.learn.libsvm.Trainer(machine_type='ONE_CLASS', kernel_type='RBF', probability=True)
    setattr(self.trainer, 'nu', self.nu)
    setattr(self.trainer, 'gamma', self.gamma)


  def train_projector(self, training_features, projector_file):
    """
    Trains the One Class SVM

    **Parameters**

    training_features:
    """
    # training_features[0] - training features for the REAL class.
    # training_features[1] - training features for the ATTACK class.
    
    # The data - "positive class only"
    pos = numpy.array(training_features[0])

    if self.rescale:
      for i in range(pos.shape[0]):
        min_value = numpy.min(pos[i])
        max_value = numpy.max(pos[i])
        pos[i] = ((2 * (pos[i] - min_value))/ (max_value - min_value)) - 1
    
    pos = [pos]
    
    # train
    self.machine = self.trainer.train(pos)
    f = bob.io.base.HDF5File(projector_file, 'w')
    self.machine.save(f)
    

  def project(self, feature):
    """
    Project the given feature
    """
    if self.rescale:
      min_value = numpy.min(feature)
      max_value = numpy.max(feature)
      feature = ((2 * (feature - min_value))/ (max_value - min_value)) - 1
    
    return self.machine(feature)

  def score(self, toscore):
    return [toscore[0]]
